@@ -2,12 +2,15 @@
 
 import base64
 from enum import Enum
-from typing import Dict, List, Optional, Union
+from typing import Any, Dict, List, Optional, Union
 
-from authlib.jose import errors as jwt_errors, jwk, jwt
+from authlib import __version__ as authlib_version
+from authlib.jose import errors as jwt_errors, JsonWebKey, jwk, jwt
 from authlib.jose.util import extract_header
+from cachetools import TTLCache
 from cryptography.hazmat.primitives import serialization
 import msal
+from packaging import version
 from pkg_resources import resource_string
 from pydantic import BaseSettings as _BaseSettings, Field, HttpUrl, PrivateAttr, SecretStr, validator
 import requests
@@ -85,7 +88,11 @@ class AADSessionAuthenticator(SessionAuthenticator):
                 port = ''
             else:
                 port = f':{request.url.port}'
-            redirect_uri = f'{request.url.scheme}://{request.url.hostname}{port}{self._redirect_path}'
+            if 'X-Forwarded-Proto' in request.headers and request.headers['X-Forwarded-Proto'] == 'https':
+                url_scheme = 'https'
+            else:
+                url_scheme = 'http'
+            redirect_uri = f'{url_scheme}://{request.url.hostname}{port}{self._redirect_path}'
         self.logger.info(f'Created redirect uri: {redirect_uri} from {request.url}')
         return redirect_uri
 
@@ -163,31 +170,51 @@ class AADTokenValidator(TokenValidator):
                  strict: bool = True,
                  client_app_ids: Optional[List[str]] = None,
                  user_klass: type = User,
-                 flow_type: OAuthFlowType = OAuthFlowType.authorizationCode):  # type: ignore
+                 flow_type: OAuthFlowType = OAuthFlowType.authorizationCode,  # type: ignore
+                 roles: Optional[List[str]] = None,
+                 jwks_cache_ttl: int = 0):
         """Initialise validator for AAD token based authentication."""
         authorization_url = f"https://login.microsoftonline.com/{tenant_id}/oauth2/v2.0/authorize"
         token_url = f"https://login.microsoftonline.com/{tenant_id}/oauth2/v2.0/token"
         self.key_url = f"https://login.microsoftonline.com/{tenant_id}/discovery/v2.0/keys"
         self.tenant_id = tenant_id
+        self._roles = roles
         super().__init__(client_id=client_id, authorizationUrl=authorization_url, tokenUrl=token_url, api_audience=api_audience, scheme_name=scheme_name,
                          scopes=scopes, auto_error=auto_error, enabled=enabled, use_pkce=use_pkce, user_klass=user_klass, flow_type=flow_type)
         self.strict = strict
         if client_app_ids is None:
             client_app_ids = []
         self.client_app_ids = client_app_ids
+        self._jwks_cache: TTLCache = TTLCache(maxsize=1, ttl=jwks_cache_ttl)
 
-    def _get_ms_jwk(self, token):
+    def _request_ms_jwks(self) -> Dict[str, Any]:
+        if len(self._jwks_cache):
+            jwks = self._jwks_cache[None]
+        else:
+            jwks = requests.get(self.key_url).json()
+            self._jwks_cache[None] = jwks
+        return jwks
+
+    def _get_ms_jwk(self, token: str) -> JsonWebKey:
         try:
             self.logger.info(f'Getting signing keys from {self.key_url}')
-            jwks = requests.get(self.key_url).json()
             token_header = token.split(".")[0].encode()
+            jwks = self._request_ms_jwks()
             unverified_header = extract_header(token_header, jwt_errors.DecodeError)
-            for key in jwks["keys"]:
-                if key["kid"] == unverified_header["kid"]:
-                    self.logger.info(f'Identified key {key["kid"]}')
-                    return jwk.loads(key)
+            # Authlib 1.0.0 changed behaviour of loads in 1.0.0
+            if version.parse(authlib_version) >= version.parse('1.0.0'):
+                # This can now raise a ValueError: Invalid JSON Web Key Set if
+                # the key is not found
+                return jwk.loads(jwks, unverified_header['kid'])
+            else:
+                for key in jwks["keys"]:
+                    if key["kid"] == unverified_header["kid"]:
+                        self.logger.info(f'Identified key {key["kid"]}')
+                        return jwk.loads(key)
         except jwt_errors.DecodeError:
             self.logger.exception('Error parsing signing keys')
+        except ValueError:
+            self.logger.exception('Error finding key')
         raise AuthenticationError("Unable to parse signing keys")
 
     def _decode_token(self, token):
@@ -195,7 +222,13 @@ class AADTokenValidator(TokenValidator):
         claims = None
         self.logger.debug(f'Key is {jwk_}')
         try:
-            if hasattr(jwk, 'public_bytes'):
+            if hasattr(jwk_, 'as_pem'):
+                # Authlib 1.0.0
+                key = jwk_.as_pem()
+            elif hasattr(jwk_, 'public_key'):
+                # Authlib 1.0.0
+                key = jwk_.public_key.public_bytes(serialization.Encoding.PEM, serialization.PublicFormat.PKCS1)
+            elif hasattr(jwk_, 'public_bytes'):
                 key = jwk_.public_bytes(serialization.Encoding.PEM, serialization.PublicFormat.PKCS1)
             else:
                 key = jwk_.raw_key.public_bytes(serialization.Encoding.PEM, serialization.PublicFormat.PKCS1)
@@ -230,6 +263,12 @@ class AADTokenValidator(TokenValidator):
     @property
     def _claims_options(self):
         options = super()._claims_options
+        if self._roles:
+            roles = self._roles
+            if not isinstance(roles, str):
+                # This is a space delimited string of roles in the claims
+                roles = ' '.join(self._roles)
+            options['roles'] = roles
         options["iss"] = {"essential": True, "values": [f"https://sts.windows.net/{self.tenant_id}/", f"https://login.microsoftonline.com/{self.tenant_id}/v2.0"]}
         options["aud"] = {"essential": True, "values": [self.api_audience] + [self.client_id] + self.client_app_ids}
         options["azp"] = {"essential": True, "values": [self.client_id] + self.client_app_ids}
@@ -272,7 +311,9 @@ class AADProvider(Provider):
             oauth_base_route: str = '/oauth',
             token_type: Union[str, TokenType] = TokenType.access,
             token_scopes: Optional[Dict[str, str]] = None,
-            flow_type: OAuthFlowType = OAuthFlowType.authorizationCode):  # type: ignore
+            flow_type: OAuthFlowType = OAuthFlowType.authorizationCode,  # type: ignore
+            roles: Optional[List[str]] = None,
+            jwks_cache_ttl: int = 0):
         """Initialise the auth backend.
 
         Args:
@@ -298,7 +339,7 @@ class AADProvider(Provider):
             token_scopes = {}
         token_validator = AADTokenValidator(client_id=client_id, tenant_id=tenant_id, api_audience=api_audience,
                                             client_app_ids=client_app_ids, scopes=token_scopes, enabled=enabled, strict=strict_token,
-                                            user_klass=user_klass, flow_type=flow_type)
+                                            user_klass=user_klass, flow_type=flow_type, roles=roles, jwks_cache_ttl=jwks_cache_ttl)
         session_authenticator = AADSessionAuthenticator(session_validator=session_validator, token_validator=token_validator,
                                                         client_id=client_id, tenant_id=tenant_id, redirect_path=redirect_path,
                                                         prompt=prompt, client_secret=client_secret, scopes=scopes,
@@ -331,7 +372,9 @@ class AADProvider(Provider):
                   prompt=provider_config.prompt, domain_hint=provider_config.domain_hint,
                   redirect_uri=provider_config.redirect_uri, user_klass=user_klass,
                   oauth_base_route=config.routing.oauth_base_route, token_type=provider_config.token_type,
-                  token_scopes=provider_config.token_scopes, flow_type=provider_config.flow_type)
+                  token_scopes=provider_config.token_scopes, flow_type=provider_config.flow_type,
+                  roles=provider_config.roles,
+                  jwks_cache_ttl=provider_config.jwks_cache_ttl)
         # We need to override the login and redirect etc until it is deprecated
         if hasattr(config.routing, 'login_path') and config.routing.login_path and not is_deprecated(config.routing.__fields__['login_path']):
             obj._login_url = config.routing.login_path
@@ -373,6 +416,7 @@ class AADConfig(ProviderConfig):
     domain_hint: Optional[str] = Field(None, description="AAD domain hint", env='AAD_DOMAIN_HINT')
     roles: Optional[List[str]] = Field(None, description="AAD roles required in claims", env='AAD_ROLES')
     token_type: TokenType = Field(TokenType.access, description='The AAD token type to use to validate (we should use the access token if it is configured, unless we are acting as a pure UI component')
+    jwks_cache_ttl: int = Field(0, description='Cache Time to Live value for MS JWKs token', env='AAD_JWKS_CACHE_TTL')
     _provider_klass: type = PrivateAttr(AADProvider)
 
     class Config:  # noqa D106
